@@ -54,8 +54,8 @@ where
             }
         })
         .filter(None, LevelFilter::Trace)
-        .parse(filter)
-        .parse(&env::var("RUST_LOG").unwrap_or_else(|_| "".to_owned()))
+        .parse_filters(filter)
+        .parse_env("RUST_LOG")
         .init();
 }
 
