@@ -1,5 +1,5 @@
-use core::fmt;
 use crate::{Error, Result};
+use core::fmt;
 
 use crate::wire::IpProtocol as Protocol;
 use crate::wire::Ipv6Address as Address;
@@ -37,22 +37,23 @@ enum_with_unknown! {
 impl fmt::Display for Type {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Type::Type0            => write!(f, "Type0"),
-            Type::Nimrod           => write!(f, "Nimrod"),
-            Type::Type2            => write!(f, "Type2"),
-            Type::Rpl              => write!(f, "Rpl"),
-            Type::Experiment1      => write!(f, "Experiment1"),
-            Type::Experiment2      => write!(f, "Experiment2"),
-            Type::Reserved         => write!(f, "Reserved"),
-            Type::Unknown(id)      => write!(f, "{}", id)
+            Type::Type0 => write!(f, "Type0"),
+            Type::Nimrod => write!(f, "Nimrod"),
+            Type::Type2 => write!(f, "Type2"),
+            Type::Rpl => write!(f, "Rpl"),
+            Type::Experiment1 => write!(f, "Experiment1"),
+            Type::Experiment2 => write!(f, "Experiment2"),
+            Type::Reserved => write!(f, "Reserved"),
+            Type::Unknown(id) => write!(f, "{}", id),
         }
     }
 }
 
 /// A read/write wrapper around an IPv6 Routing Header buffer.
 #[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct Header<T: AsRef<[u8]>> {
-    buffer: T
+    buffer: T,
 }
 
 // Format of the Routing Header
@@ -75,15 +76,15 @@ mod field {
     use crate::wire::field::*;
 
     // Minimum size of the header.
-    pub const MIN_HEADER_SIZE:  usize = 4;
+    pub const MIN_HEADER_SIZE: usize = 4;
 
     // 8-bit identifier of the header immediately following this header.
-    pub const NXT_HDR:  usize = 0;
+    pub const NXT_HDR: usize = 0;
     // 8-bit unsigned integer. Length of the DATA field in 8-octet units,
     // not including the first 8 octets.
-    pub const LENGTH:   usize = 1;
+    pub const LENGTH: usize = 1;
     // 8-bit identifier of a particular Routing header variant.
-    pub const TYPE:     usize = 2;
+    pub const TYPE: usize = 2;
     // 8-bit unsigned integer. The number of route segments remaining.
     pub const SEG_LEFT: usize = 3;
     // Variable-length field. Routing-Type-specific data.
@@ -129,9 +130,9 @@ mod field {
     // +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
     // 8-bit field containing the CmprI and CmprE values.
-    pub const CMPR:  usize = 4;
+    pub const CMPR: usize = 4;
     // 8-bit field containing the Pad value.
-    pub const PAD:   usize = 5;
+    pub const PAD: usize = 5;
     // Variable length field containing addresses
     pub fn ADDRESSES(length_field: u8) -> Field {
         let data = DATA(length_field);
@@ -233,7 +234,6 @@ impl<T: AsRef<[u8]>> Header<T> {
         data[field::CMPR] >> 4
     }
 
-
     /// Return the number of prefix octects elided from the last address (`addresses[n]`).
     ///
     /// # Panics
@@ -315,7 +315,7 @@ impl<T: AsRef<[u8]> + AsMut<[u8]>> Header<T> {
                 data[7] = 0;
             }
 
-            _ => panic!("Unrecognized routing type when clearing reserved fields.")
+            _ => panic!("Unrecognized routing type when clearing reserved fields."),
         }
     }
 }
@@ -389,63 +389,62 @@ impl<'a, T: AsRef<[u8]> + ?Sized> fmt::Display for Header<&'a T> {
 
 /// A high-level representation of an IPv6 Routing Header.
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[non_exhaustive]
 pub enum Repr<'a> {
     Type2 {
         /// The type of header immediately following the Routing header.
-        next_header:    Protocol,
+        next_header: Protocol,
         /// Length of the Routing header in 8-octet units, not including the first 8 octets.
-        length:         u8,
+        length: u8,
         /// Number of route segments remaining.
-        segments_left:  u8,
+        segments_left: u8,
         /// The home address of the destination mobile node.
         home_address: Address,
     },
     Rpl {
         /// The type of header immediately following the Routing header.
-        next_header:    Protocol,
+        next_header: Protocol,
         /// Length of the Routing header in 8-octet units, not including the first 8 octets.
-        length:         u8,
+        length: u8,
         /// Number of route segments remaining.
-        segments_left:  u8,
+        segments_left: u8,
         /// Number of prefix octets from each segment, except the last segment, that are elided.
-        cmpr_i:         u8,
+        cmpr_i: u8,
         /// Number of prefix octets from the last segment that are elided.
-        cmpr_e:         u8,
+        cmpr_e: u8,
         /// Number of octets that are used for padding after `address[n]` at the end of the
         /// RPL Source Route Header.
-        pad:            u8,
+        pad: u8,
         /// Vector of addresses, numbered 1 to `n`.
-        addresses:      &'a[u8],
+        addresses: &'a [u8],
     },
 }
 
-
 impl<'a> Repr<'a> {
     /// Parse an IPv6 Routing Header and return a high-level representation.
-    pub fn parse<T>(header: &'a Header<&'a T>) -> Result<Repr<'a>> where T: AsRef<[u8]> + ?Sized {
+    pub fn parse<T>(header: &'a Header<&'a T>) -> Result<Repr<'a>>
+    where
+        T: AsRef<[u8]> + ?Sized,
+    {
         match header.routing_type() {
-            Type::Type2 => {
-                Ok(Repr::Type2 {
-                    next_header: header.next_header(),
-                    length: header.header_len(),
-                    segments_left: header.segments_left(),
-                    home_address: header.home_address(),
-                })
-            }
-            Type::Rpl => {
-                Ok(Repr::Rpl {
-                    next_header: header.next_header(),
-                    length: header.header_len(),
-                    segments_left: header.segments_left(),
-                    cmpr_i: header.cmpr_i(),
-                    cmpr_e: header.cmpr_e(),
-                    pad: header.pad(),
-                    addresses: header.addresses(),
-                })
-            }
+            Type::Type2 => Ok(Repr::Type2 {
+                next_header: header.next_header(),
+                length: header.header_len(),
+                segments_left: header.segments_left(),
+                home_address: header.home_address(),
+            }),
+            Type::Rpl => Ok(Repr::Rpl {
+                next_header: header.next_header(),
+                length: header.header_len(),
+                segments_left: header.segments_left(),
+                cmpr_i: header.cmpr_i(),
+                cmpr_e: header.cmpr_e(),
+                pad: header.pad(),
+                addresses: header.addresses(),
+            }),
 
-            _ => Err(Error::Unrecognized)
+            _ => Err(Error::Unrecognized),
         }
     }
 
@@ -453,16 +452,19 @@ impl<'a> Repr<'a> {
     /// representation.
     pub fn buffer_len(&self) -> usize {
         match self {
-            &Repr::Rpl { length, .. } | &Repr::Type2 { length, .. } => {
-                field::DATA(length).end
-            }
+            &Repr::Rpl { length, .. } | &Repr::Type2 { length, .. } => field::DATA(length).end,
         }
     }
 
     /// Emit a high-level representation into an IPv6 Routing Header.
     pub fn emit<T: AsRef<[u8]> + AsMut<[u8]> + ?Sized>(&self, header: &mut Header<&mut T>) {
         match *self {
-            Repr::Type2 { next_header, length, segments_left, home_address } => {
+            Repr::Type2 {
+                next_header,
+                length,
+                segments_left,
+                home_address,
+            } => {
                 header.set_next_header(next_header);
                 header.set_header_len(length);
                 header.set_routing_type(Type::Type2);
@@ -470,7 +472,15 @@ impl<'a> Repr<'a> {
                 header.clear_reserved();
                 header.set_home_address(home_address);
             }
-            Repr::Rpl { next_header, length, segments_left, cmpr_i, cmpr_e, pad, addresses } => {
+            Repr::Rpl {
+                next_header,
+                length,
+                segments_left,
+                cmpr_i,
+                cmpr_e,
+                pad,
+                addresses,
+            } => {
                 header.set_next_header(next_header);
                 header.set_header_len(length);
                 header.set_routing_type(Type::Rpl);
@@ -488,11 +498,31 @@ impl<'a> Repr<'a> {
 impl<'a> fmt::Display for Repr<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Repr::Type2 { next_header, length, segments_left, home_address } => {
-                write!(f, "IPv6 Routing next_hdr={} length={} type={} seg_left={} home_address={}",
-                       next_header, length, Type::Type2, segments_left, home_address)
+            Repr::Type2 {
+                next_header,
+                length,
+                segments_left,
+                home_address,
+            } => {
+                write!(
+                    f,
+                    "IPv6 Routing next_hdr={} length={} type={} seg_left={} home_address={}",
+                    next_header,
+                    length,
+                    Type::Type2,
+                    segments_left,
+                    home_address
+                )
             }
-            Repr::Rpl { next_header, length, segments_left, cmpr_i, cmpr_e, pad, .. } => {
+            Repr::Rpl {
+                next_header,
+                length,
+                segments_left,
+                cmpr_i,
+                cmpr_e,
+                pad,
+                ..
+            } => {
                 write!(f, "IPv6 Routing next_hdr={} length={} type={} seg_left={} cmpr_i={} cmpr_e={} pad={}",
                        next_header, length, Type::Rpl, segments_left, cmpr_i, cmpr_e, pad)
             }
@@ -505,12 +535,10 @@ mod test {
     use super::*;
 
     // A Type 2 Routing Header
-    static BYTES_TYPE2: [u8; 24] = [0x6, 0x2, 0x2, 0x1,
-                                    0x0, 0x0, 0x0, 0x0,
-                                    0x0, 0x0, 0x0, 0x0,
-                                    0x0, 0x0, 0x0, 0x0,
-                                    0x0, 0x0, 0x0, 0x0,
-                                    0x0, 0x0, 0x0, 0x1];
+    static BYTES_TYPE2: [u8; 24] = [
+        0x6, 0x2, 0x2, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+        0x0, 0x0, 0x0, 0x0, 0x0, 0x1,
+    ];
 
     // A representation of a Type 2 Routing header
     static REPR_TYPE2: Repr = Repr::Type2 {
@@ -521,16 +549,11 @@ mod test {
     };
 
     // A Source Routing Header with full IPv6 addresses in bytes
-    static BYTES_SRH_FULL: [u8; 40] = [0x6,  0x4, 0x3, 0x2,
-                                       0x0,  0x0, 0x0, 0x0,
-                                       0xfd, 0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x0, 0x2,
-                                       0xfd, 0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x0, 0x0,
-                                       0x0,  0x0, 0x3, 0x1];
+    static BYTES_SRH_FULL: [u8; 40] = [
+        0x6, 0x4, 0x3, 0x2, 0x0, 0x0, 0x0, 0x0, 0xfd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+        0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0xfd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+        0x0, 0x0, 0x3, 0x1,
+    ];
 
     // A representation of a Source Routing Header with full IPv6 addresses
     static REPR_SRH_FULL: Repr = Repr::Rpl {
@@ -540,21 +563,16 @@ mod test {
         cmpr_i: 0,
         cmpr_e: 0,
         pad: 0,
-        addresses: &[0xfd, 0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x0, 0x2,
-                     0xfd, 0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x0, 0x0,
-                     0x0,  0x0, 0x3, 0x1]
+        addresses: &[
+            0xfd, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0xfd,
+            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0x1,
+        ],
     };
 
     // A Source Routing Header with elided IPv6 addresses in bytes
-    static BYTES_SRH_ELIDED: [u8; 16] = [0x6,  0x1,  0x3,  0x2,
-                                         0xfe, 0x50, 0x0,  0x0,
-                                         0x2,  0x3,  0x1,  0x0,
-                                         0x0,  0x0,  0x0,  0x0];
+    static BYTES_SRH_ELIDED: [u8; 16] = [
+        0x6, 0x1, 0x3, 0x2, 0xfe, 0x50, 0x0, 0x0, 0x2, 0x3, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0,
+    ];
 
     // A representation of a Source Routing Header with elided IPv6 addresses
     static REPR_SRH_ELIDED: Repr = Repr::Rpl {
@@ -564,20 +582,37 @@ mod test {
         cmpr_i: 15,
         cmpr_e: 14,
         pad: 5,
-        addresses: &[0x2, 0x3, 0x1, 0x0,
-                     0x0, 0x0, 0x0, 0x0]
+        addresses: &[0x2, 0x3, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0],
     };
 
     #[test]
     fn test_check_len() {
         // less than min header size
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_TYPE2[..3]).check_len());
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_SRH_FULL[..3]).check_len());
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_SRH_ELIDED[..3]).check_len());
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_TYPE2[..3]).check_len()
+        );
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_SRH_FULL[..3]).check_len()
+        );
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_SRH_ELIDED[..3]).check_len()
+        );
         // less than specfied length field
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_TYPE2[..23]).check_len());
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_SRH_FULL[..39]).check_len());
-        assert_eq!(Err(Error::Truncated), Header::new(&BYTES_SRH_ELIDED[..11]).check_len());
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_TYPE2[..23]).check_len()
+        );
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_SRH_FULL[..39]).check_len()
+        );
+        assert_eq!(
+            Err(Error::Truncated),
+            Header::new(&BYTES_SRH_ELIDED[..11]).check_len()
+        );
         // valid
         assert_eq!(Ok(()), Header::new(&BYTES_TYPE2[..]).check_len());
         assert_eq!(Ok(()), Header::new(&BYTES_SRH_FULL[..]).check_len());
